package main

import (
	"3g-mdk-go/core"
	"3g-mdk-go/dto/LinePay"
	"3g-mdk-go/dto/Search"
	"context"
	"fmt"
	"github.com/google/uuid"
	"log/slog"
	"net"
	"net/http"
	"net/http/httptest"
	"os"
)

func linepay() {
	handler := http.HandlerFunc(
		func(w http.ResponseWriter, request *http.Request) {
			_, _ = fmt.Fprintln(w, "OK")
		},
	)
	listener, _ := net.Listen("tcp", ":"+LocalServerPort)
	server := httptest.Server{
		Listener: listener,
		Config:   &http.Server{Handler: handler},
	}
	server.Start()
	defer server.Close()

	config := getConfig()

	slog.SetDefault(
		slog.New(
			core.WithWriteTraceIdHandler(
				slog.NewTextHandler(os.Stdout, &slog.HandlerOptions{}),
			),
		),
	)

	redirectionUri := "http://localhost:" + LocalServerPort + "/"

	u, _ := uuid.NewRandom()
	ctx := core.WithTraceID(context.Background(), u.String())

	orderId1 := "order-" + u.String()

	// LinePayAuthorize (Test paymentConfirmType=0)
	linePayAuthorizeRequest := &LinePay.AuthorizeRequest{
		OrderId:            orderId1,
		Amount:             "100",
		WithCapture:        "false",
		ItemName:           "サンプル商品名",
		SuccessUrl:         redirectionUri,
		CancelUrl:          redirectionUri,
		ErrorUrl:           redirectionUri,
		PaymentConfirmType: "0",
	}

	var linePayAuthorizeResponse LinePay.AuthorizeResponse
	core.Execute(linePayAuthorizeRequest, &linePayAuthorizeResponse, config, ctx)

	fmt.Printf("mstatus: %s\n", linePayAuthorizeResponse.Mstatus)
	fmt.Printf("vResultCode: %s\n", linePayAuthorizeResponse.VResultCode)

	if linePayAuthorizeResponse.RedirectWebUrl == "" {
		fmt.Printf("RedirectWebUrl is empty")
		os.Exit(-1)
	}

	pw, browser, page := openBrowser()
	_, _ = page.Goto(linePayAuthorizeResponse.RedirectWebUrl)
	_ = page.WaitForURL("*/dummy-linepay/*")
	_ = page.Locator("input[type=\"submit\"]").Click()
	_ = page.Locator("input[type=\"submit\"][value=\"購入\"]").Click()
	_ = page.WaitForURL(redirectionUri + "*")
	closeBrowser(pw, browser, page)

	u, _ = uuid.NewRandom()
	ctx = core.WithTraceID(context.Background(), u.String())

	// Search
	searchRequest := &Search.Request{
		ServiceTypeCd:    []string{"linepay"},
		ContainDummyFlag: "true",
		SearchParameters: &Search.Parameters{
			Common: &Search.CommonSearchParameter{
				OrderId: orderId1,
			},
		},
	}

	var searchResponse Search.Response
	core.Execute(searchRequest, &searchResponse, config, ctx)

	fmt.Printf("mstatus: %s\n", searchResponse.Mstatus)
	fmt.Printf("vResultCode: %s\n", searchResponse.VResultCode)
	fmt.Printf("searchCount: %v\n", searchResponse.SearchCount)
	fmt.Printf("overMaxCountFlag: %v\n", searchResponse.OverMaxCountFlag)

	u, _ = uuid.NewRandom()
	ctx = core.WithTraceID(context.Background(), u.String())

	// LinePayCapture
	captureRequest := &LinePay.CaptureRequest{
		OrderId: orderId1,
	}

	var captureResponse LinePay.CaptureResponse
	core.Execute(captureRequest, &captureResponse, config, ctx)

	fmt.Printf("mstatus: %s\n", captureResponse.Mstatus)
	fmt.Printf("vResultCode: %s\n", captureResponse.VResultCode)

	u, _ = uuid.NewRandom()
	ctx = core.WithTraceID(context.Background(), u.String())

	// LinePayCancel
	cancelRequest := &LinePay.CancelRequest{
		OrderId: orderId1,
	}

	var cancelResponse LinePay.CancelResponse
	core.Execute(cancelRequest, &cancelResponse, config, ctx)

	fmt.Printf("mstatus: %s\n", cancelResponse.Mstatus)
	fmt.Printf("vResultCode: %s\n", cancelResponse.VResultCode)

	u, _ = uuid.NewRandom()
	ctx = core.WithTraceID(context.Background(), u.String())

	orderId2 := "order-" + u.String()

	// LinePayAuthorize (Test paymentConfirmType=1)
	linePayAuthorizeRequest2 := &LinePay.AuthorizeRequest{
		OrderId:            orderId2,
		Amount:             "100",
		WithCapture:        "false",
		ItemName:           "サンプル商品名",
		PaymentConfirmType: "1",
		OneTimeKey:         "abcdef0123456789abc",
	}

	var linePayAuthorizeResponse2 LinePay.AuthorizeResponse
	core.Execute(linePayAuthorizeRequest2, &linePayAuthorizeResponse2, config, ctx)

	fmt.Printf("mstatus: %s\n", linePayAuthorizeResponse2.Mstatus)
	fmt.Printf("vResultCode: %s\n", linePayAuthorizeResponse2.VResultCode)

	os.Exit(0)
}
